/*******************************************************************************
** INTEL CONFIDENTIAL
**
** Copyright (c) 2020-2023 Intel Corporation All Rights Reserved
**
** This software and the related documents are Intel copyrighted materials,
** and your use of them is governed by the express license under which they
** were provided to you ("License"). Unless the License provides otherwise,
** you may not use, modify, copy, publish, distribute, disclose or transmit
** this software or the related documents without Intel's prior written
** permission.
**
** The source code contained or described herein and all documents related to
** the source code ("Material") are owned by Intel Corporation or its suppliers
** or licensors.
**
** This software and the related documents are provided as is,
** with no express or implied warranties, other than those that are expressly
** stated in the License.
**
*******************************************************************************/

#include <iostream>
#include <memory>
#include <string>
#include <vector>

#include "Exceptions.h"
#include "JsonFields.h"
#include "ProviderDelegate.h"
#include "SampleBatteryProvider.h"

using Json = nlohmann::json;
using std::string;

// Mocked sample driver.
class MockedBatteryDriver
{
private:
	string x = "charging";

private:
	int batteryPercentage = 90;

public:
	string getState()
	{
		return x;
	}

public:
	void setState(string value)
	{
		x = value;
	}

public:
	void takeAction(Json& result)
	{
		result["success"] = true;
		result["error"] = "None";
	}

public:
	int getBatteryPercentage()
	{
		return batteryPercentage;
	}
};

static const std::string kDataSchema = R"(
{
    "$schema": "http://json-schema.org/draft-07/schema#",
    "type": "object",
    "properties": {
        "manufacturer": {
            "type": "string",
            "readOnly": "false"
        },
        "model": {
            "type": "string",
            "readOnly": "false"
        },
        "state": {
            "type": "string",
            "enum": ["charged", "charging"]
        },
        "is_ion_battery": {
            "type": "boolean",
            "readOnly": "false"
        },
        "childCell": {
            "type": "object",
            "properties": {
                "state": {
                    "type": "string",
                    "enum": ["charged", "charging"]
                }
            }
        }
    },
    "required": [
        "state"
    ]
}
)";

static const std::string kEventSchema = R"(
{
  "$schema": "http://json-schema.org/draft-07/schema",
  "type": "object",
  "properties": {
    "batteryFullyCharged": {
      "$comment": "This is the event description raised by IPF framework.",
        "$id": "#/battery/batteryFullyCharged",
        "type": "object",
        "properties": {
          "state": {
            "type": "string",
            "enum": ["charged","charging"]
          }
        },
        "additionalProperties": false,
        "required": [ "state" ]
     },
    "batteryAttach": {
      "$id": "#/root/battery/batteryAttach",
      "type": "object",
      "properties": {
        "instance_name": {
          "type": "string"
        }
      },
      "additionalProperties": false,
      "required": [ "instance_name" ]
    },
    "batteryDetach": {
      "$id": "#/root/battery/displayDetach",
      "type": "object",
      "properties": {
        "instance_name": {
          "type": "string"
        }
      },
      "additionalProperties": false,
      "required": [ "instance_name" ]
    }
  },
  "additionalProperties": false
}
)";

static const std::string kCommandSchema = R"(
{
    "$schema": "http://json-schema.org/draft-07/schema",
    "type": "object",
    "properties": {
        "commandName": {
            "type": "string"
        },
        "simulateUnplug": {
          "$id": "#/root/battery/simulateUnplug",
          "type": "object",
          "properties": {
            "instance_name": {
              "type": "string"
            }
          },
          "required": [ "instance_name" ]
        },
        "simulatePlug": {
          "$id": "#/root/battery/simulatePlug",
          "type": "object",
          "properties": {
            "instance_name": {
              "type": "string"
            },
            "manufacturer": {
              "type": "string"
            },
            "model":{
              "type": "string"
            }
          },
          "required": [ "instance_name", "manufacturer","model" ]
        }
    }
}
)";

static const std::string kInitSchema = R"(
{
    "$schema": "http://json-schema.org/draft-07/schema",
    "type": "object",
    "properties": {
        "manufacturer": {
            "title": "Manufacturer",
            "type": "string",
            "default": "Unknown",
            "readOnly": false,
            "examples": [
                "FooDisplays"
            ],
            "pattern": "^.*$"
        },
		"model": {
            "title": "Model",
            "type": "string",
            "default": "001",
            "readOnly": false
        }
		
    },
    "additionalProperties": false,
    "required": [
        "manufacturer"
    ]
}
)";

static const std::string kResultSchema = R"(
{
  "$schema": "http://json-schema.org/draft-07/schema",
  "type": "object",
  "properties": {
    "commandName": {
      "$id": "#/ipf/battery/commandName",
      "type": "object",
      "properties": {
        "success": {
          "type": "boolean"
        },
        "error": {
          "type": "string"
        }
      },
      "required": [ "success", "error" ]
    },
	"batteryAttach": {
      "$id": "#/root/battery/displayAttach",
      "type": "object",
      "properties": {
        "instance_name": {
          "type": "string"
        }
      },
      "additionalProperties": false,
      "required": [ "instance_name" ]
    },
    "batteryDetach": {
      "$id": "#/root/battery/displayDetach",
      "type": "object",
      "properties": {
        "instance_name": {
          "type": "string"
        }
      },
      "additionalProperties": false,
      "required": [ "instance_name" ]
    }
  }
}
)";

namespace Ipf
{
auto mockedBatteryDriver = ::MockedBatteryDriver();

static const SchemaMap kSchemaMap = {
	{SchemaType::DATA, kDataSchema},
	{SchemaType::COMMAND, kCommandSchema},
	{SchemaType::RESULT, kResultSchema},
	{SchemaType::EVENT, kEventSchema},
	{SchemaType::INIT, kInitSchema}};

SampleBatteryProvider::SampleBatteryProvider(IFramework* pfw, const std::string& config)
	: ProviderDelegate(pfw, kSchemaMap, config)
{
	// Initialize data
	namespace_["state"] = "charging";
	namespace_["childCell"]["state"] = "charged";
	namespace_["is_ion_battery"] = false;

	// Manufacturer and model are not a required parameter in the schema so we still
	// have to check that it's there in init_
	if (!config_[JsonFields::INIT].empty()) {

		if (config_[JsonFields::INIT].contains("manufacturer")) {
			namespace_["manufacturer"] = config_[JsonFields::INIT]["manufacturer"];
		}
		if (config_[JsonFields::INIT].contains("model")) {
			namespace_["model"] = config_[JsonFields::INIT]["model"];
		}
	}
	else {
		// Properties can be optionally passed in at init time
		// Define "manufacturer" and "model" property that defaults to BatteryManufacturer and 001
		// respectively but can be overriden
		namespace_["manufacturer"] = "BatteryManufacturer";
		namespace_["model"] = "001";
    }
}

SampleBatteryProvider::~SampleBatteryProvider()
{}

// UpdateNamespace is invoked every time a client calls GetNode().
void SampleBatteryProvider::UpdateNamespace(const string& path)
{
	namespace_["state"] = mockedBatteryDriver.getState();
}

void SampleBatteryProvider::SetValue(const string& path, const string& value)
{
	if (path == "state") {
		mockedBatteryDriver.setState(value);
	}
}

void SampleBatteryProvider::SetValue(const std::string& path, bool value)
{
	throw ProviderException("Set value is not supported");
}

void SampleBatteryProvider::RegisterEvent(const std::string& path)
{
	std::cout << "Battery provider got an event registration for path " << path << std::endl;
}

void SampleBatteryProvider::UnregisterEvent(const std::string& path)
{
	std::cout << "Battery provider got an event unregistration for path " << path << std::endl;
}

// Provider's ExecuteCommand() is called every time a client calls
// ExecuteCommand() Client API.
std::string SampleBatteryProvider::ExecuteCommand(const string& path, const std::string& command)
{
	Json json_command = Json::parse(command);
	Json result;
	bool err = false;
	std::string errmsg = "OK";

	if (json_command.contains("commandName")) {
		mockedBatteryDriver.takeAction(result);
	}

	if (mockedBatteryDriver.getBatteryPercentage() > 89) {
		// Events are asynchronous, but for demo, it is attached to the
		// ExecuteCommand call. Client applications can register for events with a
		// callback function. If registered, framework will execute client
		// application(s)'s callback function(s)
		Json event;
		event["batteryFullyCharged"]["state"] = "charged";
		framework_->RaiseEvent(event.dump());
	}

	// Framework is OK and command is valid
	if (json_command.contains("simulatePlug")) {
		// Create a configuration for the child display
		Json config;
		config[JsonFields::INIT]["manufacturer"] = json_command["simulatePlug"]["manufacturer"];
		config[JsonFields::INIT]["model"] = json_command["simulatePlug"]["model"];

		std::string instance_name = json_command["simulatePlug"]["instance_name"];
		framework_->CreateChildInstance("Company.Org.Battery", instance_name, config.dump());

		result["simulatePlug"] = {{"success", !err}, {"error", errmsg}};
		namespace_["instances"][instance_name] = {};

		// Fire an event to notify any listeners that a new battery is attached
		Json event;
		event["batteryAttach"]["instance_name"] = instance_name;
		framework_->RaiseEvent(event.dump());
	}

	else if (json_command.contains("simulateUnplug")) {
		std::string instance_name = json_command["simulateUnplug"]["instance_name"];
		framework_->DeleteChildInstance(instance_name);

		result["simulateUnplug"] = {{"success", !err}, {"error", errmsg}};
		namespace_["instances"].erase(instance_name);

		// Fire an event to notify any listeners that a battery was detached
		Json event;
		event["batteryDetach"]["instance_name"] = instance_name;
		framework_->RaiseEvent(event.dump());
	}

	// To be extended to include other advanced features. TBD

	return result.dump();
}
} // namespace Ipf
